
## TODO: Remove once Alba's fun is merged in CSTools
source("https://earth.bsc.es/gitlab/external/cstools/-/raw/develop-CalibrationForecast/R/CST_Calibration.R")

## Entry params data and recipe?
calibrate_datasets <- function(data, recipe) {
  hcst <- data$hcst
  obs <- data$obs
  fcst <- data$fcst
  
  # Calibration function params
  method <- recipe$Analysis$Workflow$Calibration$method
  mm <- recipe$Analysis$Datasets$Multimodel
  ncores <- 4
  na.rm <- T

  # Replicate observation array for the multi-model case
  if (mm) {
    obs.mm <- obs$data
    for(dat in 1:(dim(hcst$data)['dat'][[1]]-1)) {
      obs.mm <- abind(obs.mm, obs$data,
		      along=which(names(dim(obs$data)) == 'dat'))
    }
    names(dim(obs.mm)) <- names(dim(obs$data))
    obs$data <- obs.mm
    remove(obs.mm)
  }

  if (recipe$Analysis$Variables$freq == "monthly_mean") {
  
    CST_CALIB_METHODS <- c("bias", "evmos", "mse_min", "crps_min", "rpc-based")
    ## TODO: implement other calibration methods
    ## TODO: Restructure the code?
    if (!(method %in% CST_CALIB_METHODS)) {
      stop("Calibration method is not implemented in CSTools")
    } else {
      ## Alba's version of CST_Calibration (pending merge) is being used
      # Calibrate the hindcast and forecast
      hcst_calibrated <- CST_Calibration(hcst, obs,
                                         cal.method = method,
                                         eval.method = "leave-one-out",
                                         multi.model = mm,
                                         na.fill = TRUE,
                                         na.rm = na.rm,
                                         apply_to = NULL,
                                         alpha = NULL,
                                         memb_dim = "ensemble",
                                         sdate_dim = "syear",
                                         ncores = ncores)
      if (!is.null(fcst)) {
        # Calibrate the forecast
        fcst_calibrated <- CST_Calibration(hcst, obs, fcst,
                                           cal.method = method,
                                           eval.method = "leave-one-out",
                                           multi.model = mm,
                                           na.fill = TRUE,
                                           na.rm = na.rm,
                                           apply_to = NULL,
                                           alpha = NULL,
                                           memb_dim = "ensemble",
                                           sdate_dim = "syear",
                                           ncores = ncores)
      } else {
        fcst_calibrated <- NULL
      }

    } 

  } else if (recipe$Analysis$Variables$freq == "daily_mean") {
    # Daily data calibration using Quantile Mapping
    warning("Data is at daily frequency. Quantile Mapping will be used",
            " as the calibration method.")
    hcst_calibrated <- CST_QuantileMapping(hcst, obs,
      			             exp_cor = NULL,
      				     ## TODO: Tidy up
      			             sample_dims = c("syear",
      						     "time",
      						     "ensemble"),
      				     sample_length = NULL,
      				     method = "QUANT",
      				     ncores = ncores,
      				     na.rm = na.rm)

    if (!is.null(fcst)) {
      fcst_calibrated <- CST_QuantileMapping(hcst, obs,
      				       exp_cor = fcst,
      				       sample_dims = c("syear",
      						       "time",
      						       "ensemble"),
      				       sample_length = NULL,
      				       method = "QUANT",
      				       ncores = ncores,
      				       na.rm = na.rm)
    } else {
      fcst_calibrated <- NULL
    }
  }
  
  print("###### CALIBRATION COMPLETE ######")
  return(list(hcst_calib = hcst_calibrated, fcst_calib = fcst_calibrated))
}
